#ifdef WIN32
#pragma comment(lib, "c:\\TopfieldDisk\\DataFiles\\SDK\\TAP_SDK_DLL")
#include "TAP_Emulations.h"
#endif

#include "tap.h"
#include "skin.h"
#include "menu.h"
#include "dlmalloc.h"

bool ChangeDirSkins();

static char buffer[256];
static char comment[100];
static bool footer_defined;
static bool end_of_skin;


extern struct _skin_elements
{
	char *element_name;
	char *element_comment;
	word red;
	word green;
	word blue;
} **skin_elements;

// adjust the red, green and blue values according the severity of the gradient
void adjustRGB(byte prominent_colour, dword increment, word *r, word *g, word *b, word orig_r, word orig_g, word orig_b)
{
	switch (prominent_colour) {
		case RED:
			*r = orig_r < 128 ? ((*r + increment) > 255 ? 255 : *r + increment) : ((*r < increment) ? 0 : *r - increment);
			*g = orig_r < 128 ? ((*g + increment) > 255 ? 255 : *g + increment) : ((*g < increment) ? 0 : *g - increment);
			*b = orig_r < 128 ? ((*b + increment) > 255 ? 255 : *b + increment) : ((*b < increment) ? 0 : *b - increment);
			break;
		case GREEN:
			*r = orig_g < 128 ? ((*r + increment) > 255 ? 255 : *r + increment) : ((*r < increment) ? 0 : *r - increment);
			*g = orig_g < 128 ? ((*g + increment) > 255 ? 255 : *g + increment) : ((*g < increment) ? 0 : *g - increment);
			*b = orig_g < 128 ? ((*b + increment) > 255 ? 255 : *b + increment) : ((*b < increment) ? 0 : *b - increment);
			break;
		case BLUE:
			*r = orig_b < 128 ? ((*r + increment) > 255 ? 255 : *r + increment) : ((*r < increment) ? 0 : *r - increment);
			*g = orig_b < 128 ? ((*g + increment) > 255 ? 255 : *g + increment) : ((*g < increment) ? 0 : *g - increment);
			*b = orig_b < 128 ? ((*b + increment) > 255 ? 255 : *b + increment) : ((*b < increment) ? 0 : *b - increment);
			break;
	}
}

// create a gradient block where orig_r, orig_g & orig_b represent the base skin colour
void createGradient(word region, int width, int height, dword gradient_increment, word orig_r, word orig_g, word orig_b)
{
	word r, g, b, y;

	r = orig_r; g = orig_g; b = orig_b;

	// save a line at a time, adjusting the colour as we go
	for (y = 0; y < height; y++) {
		TAP_Osd_FillBox(region, 0, y, width+10+1, 1, RGB8888(r,g,b));
		adjustRGB(prominent_bg_colour, gradient_increment, &r, &g, &b, orig_r, orig_g, orig_b);
	}
}

void createGradients(int width, int height)
{
	// release any memory used by existing saved gradients
	if (background_gradient)			TAP_MemFree(background_gradient);
	if (highlight_gradient)				TAP_MemFree(highlight_gradient);
	if (folder_gradient)				TAP_MemFree(folder_gradient);
	if (playing_gradient)				TAP_MemFree(playing_gradient);
	if (recording_gradient)				TAP_MemFree(recording_gradient);
	if (chaseplay_gradient)				TAP_MemFree(chaseplay_gradient);
	if (skin_highlight_gradient)		TAP_MemFree(skin_highlight_gradient);
	if (options_background_gradient)	TAP_MemFree(options_background_gradient);
	if (stop_menu_background_gradient)	TAP_MemFree(stop_menu_background_gradient);
	if (options_highlight_gradient)		TAP_MemFree(options_highlight_gradient);
	if (stop_menu_highlight_gradient)	TAP_MemFree(stop_menu_highlight_gradient);
	if (auto_del_background_gradient)	TAP_MemFree(auto_del_background_gradient);
	if (auto_del_highlight_gradient)	TAP_MemFree(auto_del_highlight_gradient);
	if (keyboard_bg_gradient)			TAP_MemFree(keyboard_bg_gradient);
	if (keyboard_highlight_gradient)	TAP_MemFree(keyboard_highlight_gradient);
	if (keyboard_filler_gradient)		TAP_MemFree(keyboard_filler_gradient);

	region_gradient	= TAP_Osd_Create(0, 0, width+10+1, height, 0, OSD_Flag_MemRgn);

	// save the background gradient
	createGradient(region_gradient, width, height, gradient_increment, bg_red, bg_green, bg_blue);
	background_gradient				= TAP_Osd_SaveBox(region_gradient, 0, 0, width+10+1, height);
	options_background_gradient		= TAP_Osd_SaveBox(region_gradient, 0, 0, width-300+10+1, height);
	stop_menu_background_gradient	= TAP_Osd_SaveBox(region_gradient, 0, 0, width-400+10+1, height);
	auto_del_background_gradient	= TAP_Osd_SaveBox(region_gradient, 0, 0, width-80+10+1, height);

	// save the highlight gradient
	createGradient(region_gradient, width, height, gradient_increment, hl_red, hl_green, hl_blue);
	highlight_gradient				= TAP_Osd_SaveBox(region_gradient, 0, 0, width+10+1, height);
	skin_highlight_gradient			= TAP_Osd_SaveBox(region_gradient, 0, 0, 80-2+5+1, height);
	options_highlight_gradient		= TAP_Osd_SaveBox(region_gradient, 0, 0, width-300+10+1, height);
	stop_menu_highlight_gradient	= TAP_Osd_SaveBox(region_gradient, 0, 0, width-400+10+1, height);
	auto_del_highlight_gradient		= TAP_Osd_SaveBox(region_gradient, 0, 0, width-80+10+1, height);

	// save the folder gradient
	createGradient(region_gradient, width, height, gradient_increment, dir_red, dir_green, dir_blue);
	folder_gradient					= TAP_Osd_SaveBox(region_gradient, 0, 0, width+10+1, height);

	// save the playing gradient
	createGradient(region_gradient, width, height, gradient_increment, play_red, play_green, play_blue);
	playing_gradient				= TAP_Osd_SaveBox(region_gradient, 0, 0, width+10+1, height);	

	// save the recording gradient
	createGradient(region_gradient, width, height, gradient_increment, rec_red, rec_green, rec_blue);
	recording_gradient				= TAP_Osd_SaveBox(region_gradient, 0, 0, width+10+1, height);

	// save the chase playing gradient
	createGradient(region_gradient, width, height, gradient_increment, chase_red, chase_green, chase_blue);
	chaseplay_gradient				= TAP_Osd_SaveBox(region_gradient, 0, 0, width+10+1, height);

	// save the keyboard gradients
	TAP_Osd_Delete(region_gradient);
	region_gradient	= TAP_Osd_Create(0, 0, 60, 60, 0, OSD_Flag_MemRgn);
	createGradient(region_gradient, 60, 60, gradient_increment < 4 ? 1 : gradient_increment-2, bg_red, bg_green, bg_blue);
	keyboard_bg_gradient			= TAP_Osd_SaveBox(region_gradient, 0, 0, 60, 60);
	keyboard_filler_gradient		= TAP_Osd_SaveBox(region_gradient, 0, 0, 27, 60);
	createGradient(region_gradient, 60, 60, gradient_increment < 4 ? 1 : gradient_increment-2, hl_red, hl_green, hl_blue);
	keyboard_highlight_gradient		= TAP_Osd_SaveBox(region_gradient, 0, 0, 60, 60);

	TAP_Osd_Delete(region_gradient);
}

void createTwoTones()
{
	word r, g, b, y;

	// create a two tone background pair
	r = bg_red; g = bg_green; b = bg_blue;
	for (y = 0; y < 13; y++)
		adjustRGB(prominent_bg_colour, 1, &r, &g, &b, bg_red, bg_green, bg_blue);
	tt_body_bg_colour = RGB8888(r, g, b);

	// create a two tone highlight pair
	r = hl_red; g = hl_green; b = hl_blue;
	for (y = 0; y < 13; y++)
		adjustRGB(prominent_hl_colour, 1, &r, &g, &b, hl_red, hl_green, hl_blue);
	tt_highlight_bg_colour = RGB8888(r, g, b);

	// create a two tone folder pair
	r = dir_red; g = dir_green; b = dir_blue;
	for (y = 0; y < 13; y++)
		adjustRGB(prominent_dir_colour, 1, &r, &g, &b, dir_red, dir_green, dir_blue);
	tt_folder_bg_colour = RGB8888(r, g, b);
	
	// create a two tone playing pair
	r = play_red; g = play_green; b = play_blue;
	for (y = 0; y < 13; y++)
		adjustRGB(prominent_play_colour, 1, &r, &g, &b, play_red, play_green, play_blue);
	tt_playing_bg_colour = RGB8888(r, g, b);

	// create a two tone recording pair
	r = rec_red; g = rec_green; b = rec_blue;
	for (y = 0; y < 13; y++)
		adjustRGB(prominent_rec_colour, 1, &r, &g, &b, rec_red, rec_green, rec_blue);
	tt_recording_bg_colour = RGB8888(r, g, b);
	
	// create a two tone chase playing pair
	r = chase_red; g = chase_green; b = chase_blue;
	for (y = 0; y < 13; y++)
		adjustRGB(prominent_chase_colour, 1, &r, &g, &b, chase_red, chase_green, chase_blue);
	tt_chaseplay_bg_colour = RGB8888(r, g, b);
}

void initialiseElements()
{

	frame_colour			= RGB8888(0,0,0);
	body_bg_colour			= RGB8888(2,63,126);
	bg_red = 2; bg_green = 63; bg_blue = 126; prominent_bg_colour = BLUE;
	header_bg_colour		= RGB8888(5,7,75);
	footer_bg_colour		= RGB8888(5,7,75);
	folder_bg_colour		= RGB8888(36,56,100);
	dir_red = 36; dir_green = 56; dir_blue = 100; prominent_dir_colour = BLUE;
	highlight_bg_colour		= RGB8888(48,170,255);
	hl_red = 48; hl_green = 170; hl_blue = 255; prominent_hl_colour = BLUE;
	body_text_colour		= RGB8888(254,254,254);
	header_text_colour		= RGB8888(254,254,254);
	info_header_colour		= RGB8888(88,173,96);
	info_detail_colour		= RGB8888(173,215,198);
	folder_text_colour		= RGB8888(255,255,0);
	detail_text_colour		= RGB8888(0,250,255);
	help_text_colour		= RGB8888(204,204,0);
	highlight_text_colour	= RGB8888(254,254,254);
	footer_text_colour		= RGB8888(255,255,255);
	recording_bg_colour		= RGB8888(181,18,18);
	rec_red = 181; rec_green = 18; rec_blue = 18; prominent_rec_colour = RED;
	playing_bg_colour		= RGB8888(61,139,111);
	play_red = 61; play_green = 139; play_blue = 111; prominent_play_colour = GREEN;
	chaseplay_bg_colour		= RGB8888(255,75,15);
	chase_red = 255; chase_green = 75; chase_blue = 15; prominent_chase_colour = RED;
	disk_good_colour		= RGB8888(61,139,111);
	disk_bad_colour			= RGB8888(181,18,18);
	disk_medium_colour		= RGB8888(255,75,15);
	//recycled_colour			= RGB8888(61,139,111);
	recycled_colour			= COLOR_None;
	recording_text_colour	= RGB8888(255,255,0);
	playing_text_colour		= RGB8888(255,255,0);
	chaseplay_text_colour	= RGB8888(255,255,0);
	progress_bar_bg_colour	= RGB8888(0,0,0);
	recording_bar_colour	= RGB8888(211,18,18);
	playing_bar_colour		= RGB8888(15,226,75);
	chaseplay_bar_colour	= RGB8888(255,75,15);
	image_border_colour		= RGB8888(150,150,150);
	bare_bones_border_colour= RGB8888(122,149,255);
	bare_bones_title_colour = RGB8888(46,88,255);
	bare_bones_text_colour  = RGB8888(122,149,255);
}

bool readSkinFile(char *skin_name)
{
	TYPE_File *skin_file;
	int i;
	dword file_length;

	if (!TAP_Hdd_Exist(skin_name)) return FALSE;		
	
	skin_file = TAP_Hdd_Fopen(skin_name);
	if (skin_file == NULL) return FALSE;					

	skin_buffer = dlmalloc(8192);
	memset(skin_buffer, '\0', 8192);
	skin_pointer = 0;

	file_length = TAP_Hdd_Flen(skin_file);				
	//if ((file_length > 0)&& (file_length % 512) == 0)
	//	file_length = file_length + 512;

	TAP_Hdd_Fread(skin_buffer, file_length, 1, skin_file);

	TAP_Hdd_Fclose(skin_file);
									
	return TRUE;
}

void loadSkinElements(int gradient_width, int gradient_height)
{
	footer_defined = FALSE;
	initialiseElements();
	end_of_skin = FALSE;

	if (readSkinFile(skin_file_name)) {
		skin_pointer = 0;
		while (skin_pointer < 8192 && !end_of_skin) {
			loadSkinElement(FALSE);
		}
		dlfree(skin_buffer);
	}
	dlmalloc_trim(0);

	if (!footer_defined) footer_bg_colour = header_bg_colour;

	createGradients(gradient_width, gradient_height);
	createTwoTones();
}

void loadSkinElementsFromMemory(int number_of_elements, int gradient_width, int gradient_height)
{
	word i;

	footer_defined = FALSE;
	initialiseElements();

	for (i = 0; i < number_of_elements; i++) {
		loadSkinElementFromMemory(i);
	}

	if (!footer_defined) footer_bg_colour = header_bg_colour;

	createGradients(gradient_width, gradient_height);
	createTwoTones();
}

void freeSkinElements()
{
	int i;

	if (skin_elements) {
		for (i = 0; i < element_count; i++)
		{
			if (skin_elements[i]) 
			{
				if (skin_elements[i]->element_name)
				{
					dlfree(skin_elements[i]->element_name);
					skin_elements[i]->element_name = NULL;
				}
				if (skin_elements[i]->element_comment)
				{
					dlfree(skin_elements[i]->element_comment);
					skin_elements[i]->element_comment = NULL;
				}

				dlfree(skin_elements[i]);
				skin_elements[i] = NULL;
			}
		}

		dlfree(skin_elements);
		skin_elements = NULL;

		dlmalloc_trim(0);
	}

	element_count = 0;
}

void populateSkinElements(char *skin_name, int *number_of_elements)
{
	freeSkinElements();

	memset(comment, '\0', 100);
	end_of_skin = FALSE;

	if (readSkinFile(skin_name)) {
		skin_pointer = 0;
		while (skin_pointer < 8192 && !end_of_skin) {
			loadSkinElement(TRUE);
		}
		dlfree(skin_buffer);
	}
	dlmalloc_trim(0);

	*number_of_elements = element_count;
}

void loadSkinElement(bool save_to_array)
{
	int i;
	char *old_pointer;
	char line_string[257];
	char element_string[257];
	bool end_of_element = FALSE;

	memset(line_string, '\0', 257);
	memset(element_string, '\0', 257);
	old_pointer = (char *) skin_buffer + skin_pointer;

	i = 0;
	while (skin_pointer < 8192 && i < 256) {
		line_string[i] = skin_buffer[skin_pointer];					

		if (line_string[i] == '\0') {
			end_of_skin = TRUE;
			break;			
		}
		if (line_string[i] == '\n') break;								
		if (line_string[i] == '\r') {															
			skin_pointer++;
			break;
		}

		if (!end_of_element && line_string[i] == ' ' && (line_string[0] != '/' && line_string[0] != '\0')) {
			memcpy(element_string, line_string, i);
			element_string[i] = '\0';
			end_of_element = TRUE;
		}
			
		skin_pointer++;
		i++;
	}

	skin_buffer[skin_pointer] = '\0';		
	if (strlen(element_string) == 0 && strlen(line_string) > 0) {
		strcpy(element_string, line_string);
		element_string[i] = '\0';
	}
	skin_pointer++;

	if (save_to_array && strlen(element_string) > 2 && element_string[0] == '/' && element_string[1] == '/')
		strcpy(comment, element_string);

	if (strlen(element_string) == 0 || element_string[0] == '\0' || element_string[0] == '\r' || element_string[0] == '\n' || element_string[0] == ' ' || element_string[0] == '/')
		return;

	translateSkinElement(element_string, save_to_array);
}

void loadSkinElementFromMemory(word idx)
{
	char element_name[40];
	word r = skin_elements[idx]->red, g = skin_elements[idx]->green, b = skin_elements[idx]->blue;
	strcpy(element_name, skin_elements[idx]->element_name);

	if (strcmp(element_name, "Frame") == 0)
		frame_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "Background") == 0) {
		body_bg_colour = RGB8888(r, g, b);
		if ((r > g && r > b) || (r > g && r == b) || (r > b && r == g) || (r == g && r == b))
			prominent_bg_colour = RED;
		ef ((g > r && g > b) || (g > r && g == b))
			prominent_bg_colour = GREEN;
		else
			prominent_bg_colour = BLUE;
		bg_red = r; bg_green = g; bg_blue = b;
	}
	ef (strcmp(element_name, "TitleBackground") == 0)
		header_bg_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "FooterBackground") == 0) {
		footer_bg_colour = RGB8888(r, g, b);
		footer_defined = TRUE;
	}
	ef (strcmp(element_name, "Highlight") == 0) {
		highlight_bg_colour = RGB8888(r, g, b);
		if ((r > g && r > b) || (r > g && r == b) || (r > b && r == g) || (r == g && r == b))
			prominent_hl_colour = RED;
		ef ((g > r && g > b) || (g > r && g == b))
			prominent_hl_colour = GREEN;
		else
			prominent_hl_colour = BLUE;
		hl_red = r; hl_green = g; hl_blue = b;
	}
	ef (strcmp(element_name, "TextForeground") == 0)
		body_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "TitleForeground") == 0)
		header_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "TextPlayedForeground") == 0 || strcmp(element_name, "DirForeground") == 0)
		folder_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "InfoHeaderForeground") == 0)
		info_header_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "InfoDetailForeground") == 0)
		info_detail_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "HelpForeground") == 0)
		help_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "ColumnDetailForeground") == 0)
		detail_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "TextHighlightedForeground") == 0)
		highlight_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "FooterForeground") == 0)
		footer_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "RecordingBackground") == 0) {
		recording_bg_colour = RGB8888(r, g, b);
		if ((r > g && r > b) || (r > g && r == b) || (r > b && r == g) || (r == g && r == b))
			prominent_rec_colour = RED;
		ef ((g > r && g > b) || (g > r && g == b))
			prominent_rec_colour = GREEN;
		else
			prominent_rec_colour = BLUE;
		rec_red = r; rec_green = g; rec_blue = b;
	}
	ef (strcmp(element_name, "PlayingBackground") == 0) {
		playing_bg_colour = RGB8888(r, g, b);
		if ((r > g && r > b) || (r > g && r == b) || (r > b && r == g) || (r == g && r == b))
			prominent_play_colour = RED;
		ef ((g > r && g > b) || (g > r && g == b))
			prominent_play_colour = GREEN;
		else
			prominent_play_colour = BLUE;
		play_red = r; play_green = g; play_blue = b;
	}
	ef (strcmp(element_name, "ChasePlayingBackground") == 0) {
		chaseplay_bg_colour = RGB8888(r, g, b);
		if ((r > g && r > b) || (r > g && r == b) || (r > b && r == g) || (r == g && r == b))
			prominent_chase_colour = RED;
		ef ((g > r && g > b) || (g > r && g == b))
			prominent_chase_colour = GREEN;
		else
			prominent_chase_colour = BLUE;
		chase_red = r; chase_green = g; chase_blue = b;
	}
	ef (strcmp(element_name, "DirBackground") == 0) {
		folder_bg_colour = RGB8888(r, g, b);
		if ((r > g && r > b) || (r > g && r == b) || (r > b && r == g) || (r == g && r == b))
			prominent_dir_colour = RED;
		ef ((g > r && g > b) || (g > r && g == b))
			prominent_dir_colour = GREEN;
		else
			prominent_dir_colour = BLUE;
		dir_red = r; dir_green = g; dir_blue = b;
	}
	ef (strcmp(element_name, "DiskSpaceHigh") == 0)
		disk_good_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "DiskSpaceLow") == 0)
		disk_bad_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "DiskSpaceMedium") == 0)
		disk_medium_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "RecycledSpace") == 0)
		recycled_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "RecordingForeground") == 0)
		recording_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "PlayingForeground") == 0)
		playing_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "ChasePlayingForeground") == 0)
		chaseplay_text_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "ProgressBarBackground") == 0)
		progress_bar_bg_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "RecordingProgressBar") == 0)
		recording_bar_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "PlayingProgressBar") == 0)
		playing_bar_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "ChasePlayingProgressBar") == 0)
		chaseplay_bar_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "ImageBorder") == 0)
		image_border_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "BareBonesBorder") == 0)
		bare_bones_border_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "BareBonesTitleForeground") == 0)
		bare_bones_title_colour = RGB8888(r, g, b);
	ef (strcmp(element_name, "BareBonesTextForeground") == 0)
		bare_bones_text_colour = RGB8888(r, g, b);
}

void translateSkinElement(char *element_string, bool save_to_array)
{
	int i;
	char element_name[40], *rgb_value;

	rgb_value = (char *) strrchr(element_string, '=');

	memset(element_name, '\0', 40);
	strncat(element_name, element_string, strlen(element_string)-strlen(rgb_value));
	rgb_value++;

	if (save_to_array)
		extractRGB(element_name, rgb_value, save_to_array);
	else {
		for (i = 0; i < SKIN_ELEMENT_COUNT; i++) {
			if (strcmp(skin_element_names[i].name, element_name) == 0)
				extractRGB(element_name, rgb_value, save_to_array);
		}
	}
}

void extractRGB(char *element_name, char *rgb_value, bool save_to_array)
{
	char r[4], *g, *b, temp[15];
	word red, green, blue;
	
	b = (char *) strrchr(rgb_value, ',');
	memset(temp, '\0', 15);
	strncat(temp, rgb_value, strlen(rgb_value)-strlen(b));
	g = (char *) strrchr(temp, ',');
	memset(r, '\0', 4);
	strncat(r, rgb_value, strlen(rgb_value)-strlen(g)-strlen(b));

	b++; g++;

	if (save_to_array) {
		if (element_count == 0)
			skin_elements = dlmalloc(sizeof(*skin_elements) * 256);

		skin_elements[element_count] = dlmalloc(sizeof(*skin_elements[element_count])); 
		memset(skin_elements[element_count], 0, sizeof (*skin_elements[element_count]));

		skin_elements[element_count]->element_name = dlmalloc(40);
		skin_elements[element_count]->element_comment = dlmalloc(256);
		strcpy(skin_elements[element_count]->element_name, element_name);
		skin_elements[element_count]->red = convertRGB(r);
		skin_elements[element_count]->green = convertRGB(g);
		skin_elements[element_count]->blue = convertRGB(b);
		if (strlen(comment) > 0)
			strcpy(skin_elements[element_count]->element_comment, comment);
		else {
			word i;
			strcpy(skin_elements[element_count]->element_comment, "");
			for (i = 0; i < SKIN_ELEMENT_COUNT; i++) {
				if (strcmp(skin_element_names[i].name, skin_elements[element_count]->element_name) == 0) {
					strcpy(skin_elements[element_count]->element_comment, "// ");
					strcat(skin_elements[element_count]->element_comment, skin_element_names[i].description);
					break;
				}
			}
		}
		memset(comment, '\0', 100);
		element_count++;
	}
	else {
		if (strcmp(element_name, "Frame") == 0)
			frame_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "Background") == 0) {
				red = convertRGB(r); green = convertRGB(g); blue = convertRGB(b);
			body_bg_colour = RGB8888(red, green, blue);
			if ((red > green && red > blue) || (red > green && red == blue) || (red > blue && red == green) || (red == green && red == blue))
				prominent_bg_colour = RED;
			ef ((green > red && green > blue) || (green > red && green == blue))
				prominent_bg_colour = GREEN;
			else
				prominent_bg_colour = BLUE;
			bg_red = red; bg_green = green; bg_blue = blue;
		}
		ef (strcmp(element_name, "TitleBackground") == 0)
			header_bg_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "FooterBackground") == 0) {
			footer_bg_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
			footer_defined = TRUE;
		}
		ef (strcmp(element_name, "Highlight") == 0) {
			red = convertRGB(r); green = convertRGB(g); blue = convertRGB(b);
			highlight_bg_colour = RGB8888(red, green, blue);
			if ((red > green && red > blue) || (red > green && red == blue) || (red > blue && red == green) || (red == green && red == blue))
				prominent_hl_colour = RED;
			ef ((green > red && green > blue) || (green > red && green == blue))
				prominent_hl_colour = GREEN;
			else
				prominent_hl_colour = BLUE;
			hl_red = red; hl_green = green; hl_blue = blue;
		}
		ef (strcmp(element_name, "TextForeground") == 0)
			body_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "TitleForeground") == 0)
			header_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "TextPlayedForeground") == 0 || strcmp(element_name, "DirForeground") == 0)
			folder_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "InfoHeaderForeground") == 0)
			info_header_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "InfoDetailForeground") == 0)
			info_detail_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "HelpForeground") == 0)
			help_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "ColumnDetailForeground") == 0)
			detail_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "TextHighlightedForeground") == 0)
			highlight_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "FooterForeground") == 0)
			footer_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "RecordingBackground") == 0) {
			red = convertRGB(r); green = convertRGB(g); blue = convertRGB(b);
			recording_bg_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
			if ((red > green && red > blue) || (red > green && red == blue) || (red > blue && red == green) || (red == green && red == blue))
				prominent_rec_colour = RED;
			ef ((green > red && green > blue) || (green > red && green == blue))
				prominent_rec_colour = GREEN;
			else
				prominent_rec_colour = BLUE;
			rec_red = red; rec_green = green; rec_blue = blue;
		}
		ef (strcmp(element_name, "PlayingBackground") == 0) {
			red = convertRGB(r); green = convertRGB(g); blue = convertRGB(b);
			playing_bg_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
			if ((red > green && red > blue) || (red > green && red == blue) || (red > blue && red == green) || (red == green && red == blue))
				prominent_play_colour = RED;
			ef ((green > red && green > blue) || (green > red && green == blue))
				prominent_play_colour = GREEN;
			else
				prominent_play_colour = BLUE;
			play_red = red; play_green = green; play_blue = blue;
		}
		ef (strcmp(element_name, "ChasePlayingBackground") == 0) {
			red = convertRGB(r); green = convertRGB(g); blue = convertRGB(b);
			chaseplay_bg_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
			if ((red > green && red > blue) || (red > green && red == blue) || (red > blue && red == green) || (red == green && red == blue))
				prominent_chase_colour = RED;
			ef ((green > red && green > blue) || (green > red && green == blue))
				prominent_chase_colour = GREEN;
			else
				prominent_chase_colour = BLUE;
			chase_red = red; chase_green = green; chase_blue = blue;
		}
		ef (strcmp(element_name, "DirBackground") == 0) {
			red = convertRGB(r); green = convertRGB(g); blue = convertRGB(b);
			folder_bg_colour = RGB8888(red, green, blue);
			if ((red > green && red > blue) || (red > green && red == blue) || (red > blue && red == green) || (red == green && red == blue))
				prominent_dir_colour = RED;
			ef ((green > red && green > blue) || (green > red && green == blue))
				prominent_dir_colour = GREEN;
			else
				prominent_dir_colour = BLUE;
			dir_red = red; dir_green = green; dir_blue = blue;
		}
		ef (strcmp(element_name, "DiskSpaceHigh") == 0)
			disk_good_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "DiskSpaceLow") == 0)
			disk_bad_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "DiskSpaceMedium") == 0)
			disk_medium_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "RecycledSpace") == 0)
			recycled_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "RecordingForeground") == 0)
			recording_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "PlayingForeground") == 0)
			playing_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "ChasePlayingForeground") == 0)
			chaseplay_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "ProgressBarBackground") == 0)
			progress_bar_bg_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "RecordingProgressBar") == 0)
			recording_bar_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "PlayingProgressBar") == 0)
			playing_bar_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "ChasePlayingProgressBar") == 0)
			chaseplay_bar_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "ImageBorder") == 0)
			image_border_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "BareBonesBorder") == 0)
			bare_bones_border_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "BareBonesTitleForeground") == 0)
			bare_bones_title_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		ef (strcmp(element_name, "BareBonesTextForeground") == 0)
			bare_bones_text_colour = RGB8888(convertRGB(r), convertRGB(g), convertRGB(b));
		}
}

word convertRGB(char *value)
{
    char 	str[5];
	word	tmp;
	int i;
	
	strcpy(str, value);
	tmp = 0;
	i = 0;

	while (str[i] != '\0') {
	    tmp = (tmp * 10) + (str[i] - '0');
		i++;
	}

	return tmp;
}

void writeSkinElementFromMemory(TYPE_File *skin_file, word idx)
{
	if (strlen(skin_elements[idx]->element_comment) > 0) {
		TAP_Hdd_Fwrite((void*) skin_elements[idx]->element_comment, strlen(skin_elements[idx]->element_comment), 1, skin_file);
		TAP_Hdd_Fwrite((void*)"\r\n", strlen("\r\n"), 1, skin_file);
		byte_count += strlen(skin_elements[idx]->element_comment) + strlen("\r\n");
	}
	TAP_SPrint(buffer, "%s=%d,%d,%d\r\n", skin_elements[idx]->element_name, skin_elements[idx]->red, skin_elements[idx]->green, skin_elements[idx]->blue);
	TAP_Hdd_Fwrite((void*) buffer, strlen(buffer), 1, skin_file);
	TAP_Hdd_Fwrite((void*)"\r\n", strlen("\r\n"), 1, skin_file);
	byte_count += strlen(buffer) + strlen("\r\n");
}

void writeSkinFromMemory(char *skin_name, int number_of_elements)
{
	TYPE_File *skin_file;
	word i;

	byte_count = 0;

	if (TAP_Hdd_Exist(skin_name)) 
		TAP_Hdd_Delete(skin_name);

	TAP_Hdd_Create(skin_name, ATTR_PROGRAM);					

	skin_file = TAP_Hdd_Fopen(skin_name);
	if (skin_file == NULL) 
		return; 									

	for (i = 0; i < number_of_elements; i++) {
		writeSkinElementFromMemory(skin_file, i);
	}

	i = 0;
	memset(buffer, '\0', 512);
	while ((byte_count % 512) != 510)
	{
		strcpy(&buffer[i], " ");
		byte_count = byte_count + 1;	
		i++;
	}
	strcpy(&buffer[i], "\r");
	strcpy(&buffer[i+1], "\n");
	TAP_Hdd_Fwrite((void*) buffer, strlen(buffer), 1, skin_file);

	TAP_Hdd_Fclose(skin_file);
}

void checkForDefaultSkin()
{
	word i;

	ChangeDirProgramFiles();
	if (TAP_Hdd_Exist(DEFAULT_SKIN_NAME))
		return;

	ChangeDirSkins();	
	if (TAP_Hdd_Exist(DEFAULT_SKIN_NAME))
		return;

	writeSkinFromMemory(DEFAULT_SKIN_NAME, 0);
}