static char *skin_buffer;
static int skin_pointer;
static word element_count;
static dword byte_count;

#define DEFAULT_SKIN_NAME "Default.mcf"

typedef struct {
    char name[40];
    char description[256];
} element_names;


// table of element names and their corresponding description
static element_names skin_element_names[] = {
    {"Frame",						"Main frame (outside edge)"},
    {"Background",					"Main background"},
	{"TitleBackground",				"Background of title areas"},
	{"FooterBackground",			"Footer background"},
	{"Highlight",					"Highlighted background"},
	{"TextForeground",				"Main text"},
	{"TitleForeground",				"Text in title areas"},
	{"TextPlayedForeground",		"Text for played recordings (also used for other elements)"},
	{"InfoHeaderForeground",		"Information titles e.g. Program name in footer"},
	{"InfoDetailForeground",		"Information details e.g. Program details in footer"},
	{"HelpForeground",				"Help text titles in footer"},
	{"ColumnDetailForeground",		"Text in right hand File list columns"},
	{"TextHighlightedForeground",	"Highlighted text"},
	{"FooterForeground",			"Footer text"},
	{"RecordingBackground",			"Background of current recordings"},
	{"PlayingBackground",			"Background of current playbacks"},
	{"ChasePlayingBackground",		"Background of current recordings being played back"},
	{"DirBackground",				"Background of folders"},
	{"DirForeground",				"Text of folders"},
	{"DiskSpaceLow",				"Fill colour of lozenge when free disk space is low"},
	{"DiskSpaceMedium",				"Fill colour of lozenge when free disk space is medium"},
	{"DiskSpaceHigh",				"Fill colour of lozenge when free disk space is high"},
	{"RecycledSpace",				"Fill colour of lozenge for recycled space"},
	{"RecordingForeground",			"Text of current recordings"},
	{"PlayingForeground",			"Text of current playbacks"},
	{"ChasePlayingForeground",		"Text of current recordings being played back"},
	{"ImageBorder",					"Border of thumbnails and progress bars."},
	{"ProgressBarBackground",		"Background of the progress bar for active recordings and playbacks."},
	{"RecordingProgressBar",		"Bar for recordings in progress."},
	{"PlayingProgressBar",			"Bar for percentage of recording watched."},
	{"ChasePlayingProgressBar",		"Bar for percentage of recording watched while chase playing."},
	{"BareBonesBorder",				"Border of Bare bones display."},
	{"BareBonesTitleForeground",	"Text of track title in Bare bones display."},
	{"BareBonesTextForeground",		"Text of all other details in Bare bones display."},
	{"Frame2",						"Inner frame of programs"},
	{"LockedBackground",			"Locked background, programs in the past"},
	{"TitleForeground2",			"Some other title elements"},
	{"NowProgram",					"Current programs"},
	{"NormalProgram",				"Programs other than past or current"},
	{"NowBar",						"Line indicating now"},
	{"MoreProgram",					"Bars indictating truncated programs"},
	{"ProgressBar",					"Progress bar"},
	{"ProgressBarText",				"Text around progress bar"},
	{"NowCount",					"Now, Now + 1 etc"},
	{"ProgramTime",					"Start time of program"},
	{"Screen",						"Area surrounding OSDs in FullScreen mode"},
	{"TodaysTimers",				"Timers set for today"},
	{"IconWatchForeground",			"Text of watched recording icon"},
	{"IconWatchBackground",			"Background of watched recording icon"},
	{"IconRecordForeground",		"Text of active recording icon"},
	{"IconRecordBackground",		"Background of active recording icon"},
	{"IconPlayForeground",			"Text of playing recording icon"},
	{"IconPlayBackground",			"Background of playing recording icon"},
	{"IconChaseForeground",			"Text of chase playing recording icon"},
	{"IconChaseBackground",			"Background of chase playing recording icon"},
	{"IconFilmForeground",			"Text of film icon"},
	{"IconFilmBackground",			"Background of film icon"},
	{"IconLockForeground",			"Text of locked recording icon"},
	{"IconLockBackground",			"Background of locked recording icon"},
	{"IconWarnForeground",			"Text of warning recording icon"},
	{"IconWarnBackground",			"Background of warning recording icon"},
	{"IconLastForeground",			"Text of last watched recording icon"},
	{"IconLastBackground",			"Background of last watched recording icon"},
	{"IconMarkForeground",			"Text of marked file icon"},
	{"IconMarkBackground",			"Background of marked file icon"},
	{"IconLastRecForeground",		"Text of latest recording icon"},
	{"IconLastRecBackground",		"Background of latest recording icon"},
};

#define SKIN_ELEMENT_COUNT  (sizeof(skin_element_names) / sizeof(element_names))

word frame_colour;
word body_bg_colour;
word tt_body_bg_colour;
word header_bg_colour;
word footer_bg_colour;
word folder_bg_colour;
word tt_folder_bg_colour;
word highlight_bg_colour;
word tt_highlight_bg_colour;
word recording_bg_colour;
word tt_recording_bg_colour;
word playing_bg_colour;
word tt_playing_bg_colour;
word chaseplay_bg_colour;
word tt_chaseplay_bg_colour;
word body_text_colour;
word header_text_colour;
word footer_text_colour;
word folder_text_colour;
word info_header_colour;
word info_detail_colour;
word detail_text_colour;
word help_text_colour;
word highlight_text_colour;
word recording_text_colour;
word chaseplay_text_colour;
word playing_text_colour;
word disk_good_colour;
word disk_bad_colour;
word disk_medium_colour;
word recycled_colour;
word progress_bar_bg_colour;
word recording_bar_colour;
word playing_bar_colour;
word chaseplay_bar_colour;
word image_border_colour;
word bare_bones_border_colour;
word bare_bones_title_colour;
word bare_bones_text_colour;
//static word highlight_colour	= RGB8888(61,139,111);   the old green one

byte prominent_bg_colour;
word bg_red, bg_green, bg_blue;
byte prominent_hl_colour;
word hl_red, hl_green, hl_blue;
byte prominent_dir_colour;
word dir_red, dir_green, dir_blue;
byte prominent_play_colour;
word play_red, play_green, play_blue;
byte prominent_rec_colour;
word rec_red, rec_green, rec_blue;
byte prominent_chase_colour;
word chase_red, chase_green, chase_blue;
#define RED 0
#define GREEN 1
#define BLUE 2

word region_gradient;
byte *background_gradient;
byte *highlight_gradient;
byte *folder_gradient;
byte *playing_gradient;
byte *recording_gradient;
byte *chaseplay_gradient;
byte *skin_highlight_gradient;
byte *options_background_gradient;
byte *stop_menu_background_gradient;
byte *options_highlight_gradient;
byte *stop_menu_highlight_gradient;
byte *auto_del_background_gradient;
byte *auto_del_highlight_gradient;
byte *keyboard_bg_gradient;
byte *keyboard_highlight_gradient;
byte *keyboard_filler_gradient;

bool readSkinFile();
void loadSkinElements();
void loadSkinElement(bool save_to_array);
void translateSkinElement(char *element_string, bool save_to_array);
void extractRGB(char *element_name, char *rgb_value, bool save_to_array);
word convertRGB(char *value);
void loadSkinElementFromMemory(word idx);



